package org.jeecg.modules.kuaishou.service.impl;

import cn.com.ctop.common.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.jeecg.modules.kuaishou.entity.KuaiShouCampaignCreate;
import org.jeecg.modules.kuaishou.entity.KuaiShouCampaignTemplate;
import org.jeecg.modules.kuaishou.entity.KuaiShouCreateAppTemplate;
import org.jeecg.modules.kuaishou.entity.KuaiShouGroupCreateLog;
import org.jeecg.modules.kuaishou.mapper.KuaiShouCampaignCreateMapper;
import org.jeecg.modules.kuaishou.mapper.KuaiShouCampaignTemplateMapper;
import org.jeecg.modules.kuaishou.mapper.KuaiShouCreateAppTemplateMapper;
import org.jeecg.modules.kuaishou.mapper.KuaiShouGroupCreateLogMapper;
import org.jeecg.modules.kuaishou.service.IKuaiShouCreateService;
import org.jeecg.modules.kuaishou.service.IKuaiShouImageService;
import org.jeecg.modules.kuaishou.service.IKuaiShouVideoService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Service
public class KuaiShouCreateServiceImpl implements IKuaiShouCreateService {
    private static final Logger logger = LoggerFactory.getLogger(KuaiShouCreateServiceImpl.class);
    @Autowired
    private KuaiShouCampaignTemplateMapper campaignTemplateMapper;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;
    @Autowired
    private KuaiShouCreateAppTemplateMapper createAppTemplateMapper;
    @Autowired
    private KuaiShouGroupCreateLogMapper groupCreateLogMapper;
    @Autowired
    private ICTopOauthTokenService oauthTokenService;
    @Autowired
    private KuaiShouCampaignCreateMapper campaignCreateMapper;

    @Override
    public Map<String, Object> createGroups(KuaiShouGroupCreateLog groupCreateTemplate) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            Integer createCount = groupCreateTemplate.getCreateCount();
            if (Check.isNull(createCount) || createCount == 0) {
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("desc", "迭代次数不能为空或为0");
                return returnMap;
            }
            Long accountId = groupCreateTemplate.getAccountId();
            if (Check.isNull(accountId)) {
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("desc", "请选择快手账号");
                return returnMap;
            }

            CTopOauthToken cTopOauthToken = oauthTokenService.getOauthTokenByAccountId(String.valueOf(accountId));
            if (Check.isNull(cTopOauthToken)) {
                logger.error("未获取到token信息,accountId:{}", accountId);
                throw new Exception("未获取到token信息");
            }
            String accessToken = cTopOauthToken.getAccessToken();

            Long campaignTemplateId = groupCreateTemplate.getCampaignTemplateId(); // 广告计划模板id
            if (Check.isNull(campaignTemplateId)) {
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("desc", "请选择广告计划模板");
                return returnMap;
            }

            KuaiShouCampaignTemplate kuaiShouCampaignTemplate = campaignTemplateMapper.selectById(campaignTemplateId); // 广告计划信息
            if (Check.isNull(kuaiShouCampaignTemplate)) {
                logger.error("查询广告计划为空:模板id:{}", campaignTemplateId);
                throw new Exception("根据模板id查询创意模板信息为空");
            }

            QueryWrapper<KuaiShouCampaignCreate> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("account_id", accountId);
            queryWrapper.eq("campaign_name", kuaiShouCampaignTemplate.getCampaignName());
            queryWrapper.orderByDesc("create_time");
            queryWrapper.last("limit 1");
            KuaiShouCampaignCreate campaignCreate = campaignCreateMapper.selectOne(queryWrapper);
            Long campaignId;
            if (Check.isNull(campaignCreate) || Check.isNull(campaignCreate.getCampaignId())) {
                JSONObject campaignJson = new JSONObject();
                campaignJson.put("campaign_name", kuaiShouCampaignTemplate.getCampaignName());
                campaignJson.put("type", kuaiShouCampaignTemplate.getType());
                campaignJson.put("day_budget", kuaiShouCampaignTemplate.getDayBudget());
                Map<String, Object> campaignMap = kuaishouInterfaceService.campaignCreate(accessToken, accountId, campaignJson);
                Integer code = (Integer) campaignMap.get("code");
                if (code != 0) {
                    returnMap.put("code", -1);
                    returnMap.put("success", false);
                    returnMap.put("desc", "创建广告计划失败");
                    return returnMap;
                }
                campaignId = (Long) campaignMap.get("campaign_id");
            } else {
                campaignId = campaignCreate.getCampaignId();
            }

            groupCreateTemplate.setCampaignId(campaignId);

            JSONObject groupJson = new JSONObject();
            Integer type = kuaiShouCampaignTemplate.getType();// 计划类型
            if (type == 3 || type == 4 || type == 5) {
                if (type == 3) {
                    groupJson.put("url_type", kuaiShouCampaignTemplate.getUrlType());
                }
                groupJson.put("url", kuaiShouCampaignTemplate.getUrl());
            }
            if (type == 2) { // app推广
                KuaiShouCreateAppTemplate appTemplate = createAppTemplateMapper.selectById(kuaiShouCampaignTemplate.getAppId());
                if (Check.isNull(appTemplate)) {
                    logger.error("快手上传视频未获取到本地app信息,appId:{}", kuaiShouCampaignTemplate.getAppId());
                    throw new Exception("快手上传视频未获取到本地app信息");
                }
                JSONObject apkJson = new JSONObject();
                String platform = appTemplate.getPlatform();
                apkJson.put("platform", platform);
                apkJson.put("app_name", appTemplate.getAppName());
                apkJson.put("app_version", appTemplate.getAppVersion());
                if (!Check.isNull(appTemplate.getPackageName())) {
                    apkJson.put("package_name", appTemplate.getPackageName());
                }
                if (!Check.isNull(appTemplate.getImageUrl()) && ("1".equals(platform) || "3".equals(platform))) {
                    Map<String, Object> imageMap = kuaishouInterfaceService.imageUpload(accountId, accessToken, (String) appTemplate.getImageUrl(), 1);
                    Integer imageCode = (Integer) imageMap.get("code");
                    if (imageCode != 0) {
                        returnMap.put("code", -1);
                        returnMap.put("success", false);
                        returnMap.put("desc", "上传图片失败");
                        return returnMap;
                    }
                    apkJson.put("image_token", imageMap.get("imageToken"));
                }

                Map<String, Object> apkMap;
                if ("1".equals(platform)) {
                    apkJson.put("file", appTemplate.getFile());
                    apkMap = kuaishouInterfaceService.appCreate(accountId, accessToken, apkJson);

                } else {
                    apkJson.put("url", appTemplate.getUrl());
                    apkMap = kuaishouInterfaceService.urlAppCreate(accountId, accessToken, apkJson);
                }
                if ((Integer) apkMap.get("code") != 0) {
                    returnMap.put("code", -1);
                    returnMap.put("success", false);
                    returnMap.put("desc", "上传apk文件失败");
                    return returnMap;
                }
                groupJson.put("app_id", apkMap.get("appId"));
            }

            groupJson.put("bid_type", groupCreateTemplate.getBidType());
            groupJson.put("campaign_id", campaignId);
            if (!Check.isNull(groupCreateTemplate.getBid())) {
                groupJson.put("bid", groupCreateTemplate.getBid());
            }
            if (!Check.isNull(groupCreateTemplate.getCpaBid())) {
                groupJson.put("cpa_bid", groupCreateTemplate.getCpaBid());
            }
            if (!Check.isNull(groupCreateTemplate.getOcpxActionType())) {
                groupJson.put("ocpx_action_type", groupCreateTemplate.getOcpxActionType());
            }
            if (!Check.isNull(groupCreateTemplate.getDeepConversionType())) {
                groupJson.put("deep_conversion_type", groupCreateTemplate.getDeepConversionType());
            }
            if (!Check.isNull(groupCreateTemplate.getPage())) {
                groupJson.put("page", groupCreateTemplate.getPage());
            }
            if (!Check.isNull(groupCreateTemplate.getBeginTime())) {
                groupJson.put("begin_time", groupCreateTemplate.getBeginTime());
            }
            if (!Check.isNull(groupCreateTemplate.getEndTime())) {
                groupJson.put("end_time", groupCreateTemplate.getEndTime());
            }
            if (!Check.isNull(groupCreateTemplate.getScheduleTime())) {
                groupJson.put("schedule_time", groupCreateTemplate.getScheduleTime());
            }
            if (!Check.isNull(groupCreateTemplate.getDayBudget())) {
                groupJson.put("day_budget", groupCreateTemplate.getDayBudget());
            }
            if (!Check.isNull(groupCreateTemplate.getShowModel())) {
                groupJson.put("show_mode", groupCreateTemplate.getShowModel());
            }
            if (!Check.isNull(groupCreateTemplate.getSpeed())) {
                groupJson.put("speed", groupCreateTemplate.getSpeed());
            }
            JSONObject targetJson = new JSONObject();
            groupJson.put("target", targetJson);

            String unitName = groupCreateTemplate.getUnitName();
            for (int i = 0; i < createCount; i++) {
                String name = unitName + "_" + i;
                groupJson.put("unit_name", name);
                Map<String, Object> groupMap = kuaishouInterfaceService.adUnitCreate(accessToken, accountId, groupJson);
                if ((Integer) groupMap.get("code") != 0) {
                    logger.error("创建广告组失败,accountId:{},unitName：{}", accountId, unitName);
                    throw new Exception("创建广告组失败");
                }
                groupCreateTemplate.setUnitId((Long) groupMap.get("unitId"));
                groupCreateTemplate.setUnitName(name);
                int y = groupCreateLogMapper.insert(groupCreateTemplate);
                if (y > 0) {
                    logger.info("创建广告组记录信息入库完成,accountId:{},unitName：{}", accountId, unitName);
                }
            }
            returnMap.put("code", 0);
            returnMap.put("success", true);
            returnMap.put("desc", "批量创建广告组成功");
        } catch (Exception e) {
            e.printStackTrace();
            returnMap.put("code", -1);
            returnMap.put("success", false);
            returnMap.put("desc", "批量创建广告组失败");

        }
        return returnMap;
    }


    /**
     * 批量创建创意
     *
     * @param requestJson
     * @return
     */
    @Autowired
    private IKuaiShouVideoService videoService;
    @Autowired
    private IKuaiShouImageService imageService;

    @Override
    public Map<String, Object> createCreative(JSONObject requestJson) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            if (Check.isNull(requestJson)) {
                logger.error("创建创意入参为空");
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "入参为空");
                return returnMap;
            }

            JSONObject createJson = new JSONObject();
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                logger.error("创建创意广告主id为空，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "请选择快手账号");
                return returnMap;
            }

            CTopOauthToken cTopOauthToken = oauthTokenService.getOauthTokenByAccountId(String.valueOf(accountId));
            if (Check.isNull(cTopOauthToken)) {
                logger.error("未获取到token信息,accountId:{}", accountId);
                throw new Exception("未获取到token信息");
            }
            String accessToken = cTopOauthToken.getAccessToken();

            Long unitId = requestJson.getLong("unitId");
            if (Check.isNull(unitId)) {
                logger.error("创建创意广告组为空，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "请选择广告组");
                return returnMap;
            }
            createJson.put("unit_id", unitId);

            String videoUrl;
            String videoUploadType = requestJson.getString("videoUploadType");
            if ("2".equals(videoUploadType)) {
                videoUrl = videoService.localInsert(requestJson);
            } else {
                videoUrl = requestJson.getString("videoUrl");
            }

            if (Check.isNull(videoUrl)) {
                logger.error("创建创意视频为空，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "请选择上传的视频");
                return returnMap;
            }

            Map<String, Object> videoMap = kuaishouInterfaceService.videoUpload(accountId, accessToken, videoUrl);
            if ((Integer) videoMap.get("code") != 0) {
                logger.error("上传视频失败，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "上传视频失败");
                return returnMap;
            }
            createJson.put("photo_id", videoMap.get("photoId"));
            if (!Check.isNull(requestJson.getString("actionBarText"))) {
                createJson.put("action_bar_text", requestJson.getString("actionBarText"));
            }

            if (Check.isNull(requestJson.getString("description"))) {
                logger.error("请填写广告语，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "请填写广告语");
                return returnMap;
            }
            createJson.put("description", requestJson.getString("description"));
            if (!Check.isNull(requestJson.getString("clickTrackUrl"))) {
                createJson.put("click_track_url", requestJson.getString("clickTrackUrl"));
            }
            JSONArray imageUrlArr = requestJson.getJSONArray("imageUrl");
            if (Check.isNull(imageUrlArr)) {
                logger.error("请选择封面图，accountId:{}", accountId);
                returnMap.put("code", -1);
                returnMap.put("success", false);
                returnMap.put("message", "请选择广告创意封面图");
                return returnMap;
            }

            String creativeName = requestJson.getString("creativeName");
            for (int i = 0; i < imageUrlArr.size(); i++) {
                String imagePath;
                String imageUrl = imageUrlArr.get(i).toString();
                if (!Check.isNull(imageUrl)) {
                    if ("2".equals(requestJson.getString("imageType"))) {
                        imagePath = imageService.localInsert(imageUrl, requestJson);
                    } else {
                        imagePath = imageUrl;
                    }

                    Map<String, Object> imageMap = kuaishouInterfaceService.imageUpload(accountId, accessToken, imagePath, 0);
                    if ((Integer) imageMap.get("code") != 0) {
                        returnMap.put("code", -1);
                        returnMap.put("success", false);
                        returnMap.put("message", "上传图片失败");
                        return returnMap;
                    }
                    createJson.put("image_token", imageMap.get("imageToken"));
                    createJson.put("creative_name", creativeName + "_" + i);
                    Map<String, Object> creativeMap = kuaishouInterfaceService.creativeCreate(accessToken, accountId, createJson);
                    if ((Integer) creativeMap.get("code") != 0) {
                        throw new Exception("批量创建创意失败");
                    }
                }

            }
            returnMap.put("code", 0);
            returnMap.put("success", true);
            returnMap.put("message", "创建创意成功");
            return returnMap;

        } catch (Exception e) {
            returnMap.put("code", -1);
            returnMap.put("success", false);
            returnMap.put("message", "批量创建广告创意失败");
            e.printStackTrace();

        }
        return returnMap;
    }


    /**
     * 三级联动 查询账号 计划 模板信息
     *
     * @param loginId
     * @return
     */
    @Override
    public JSONArray getOptions(String loginId, Long accountId) {
        JSONArray result = new JSONArray();
        QueryWrapper<KuaiShouGroupCreateLog> wrapper = new QueryWrapper<>();
        wrapper.eq("login_id", loginId).eq("account_id", accountId).orderByDesc("create_time");
        wrapper.groupBy("campaign_id");
        List<KuaiShouGroupCreateLog> accounts = groupCreateLogMapper.selectList(wrapper);
        if (!Check.isNull(accounts)) {
            for (KuaiShouGroupCreateLog groupCreateLog : accounts) {
                JSONObject optionsJson = new JSONObject();
                Long campaignId = groupCreateLog.getCampaignId();
                optionsJson.put("value", campaignId);
                QueryWrapper<KuaiShouCampaignCreate> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("account_id", accountId).eq("campaign_id", campaignId).orderByDesc("create_time");
                queryWrapper.last("limit 1");
                KuaiShouCampaignCreate campaignCreate = campaignCreateMapper.selectOne(queryWrapper);
                optionsJson.put("label", campaignCreate.getCampaignName());
                optionsJson.put("isLeaf", false);
                result.add(optionsJson);
            }

        }
        return result;
    }

    /**
     * 获取组信息
     *
     * @param loginId
     * @param campaignId
     * @return
     */
    @Override
    public JSONArray getChildren(String loginId, Long campaignId, Long accountId) {
        JSONArray result = new JSONArray();
        QueryWrapper<KuaiShouGroupCreateLog> wrapper = new QueryWrapper<>();
        wrapper.eq("account_id", accountId);
        wrapper.eq("campaign_id", campaignId);
        wrapper.eq("login_id", loginId);
        wrapper.orderByDesc("create_time");
        wrapper.groupBy("unit_name");
        List<KuaiShouGroupCreateLog> accounts = groupCreateLogMapper.selectList(wrapper);
        if (!Check.isNull(accounts)) {
            for (KuaiShouGroupCreateLog groupCreateLog : accounts) {
                JSONObject optionsJson = new JSONObject();
                optionsJson.put("value", groupCreateLog.getUnitId());
                optionsJson.put("label", groupCreateLog.getUnitName());
                result.add(optionsJson);
            }

        }
        return result;
    }
}
