package org.jeecg.modules.kuaishou.service.impl;

import cn.com.ctop.common.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import constant.KuaishouInterfaceConstant;
import org.jeecg.common.util.encryption.AesEncryptUtil;
import org.jeecg.modules.kuaishou.entity.KuaiShouImage;
import org.jeecg.modules.kuaishou.entity.KuaiShouVideo;
import org.jeecg.modules.kuaishou.mapper.KuaiShouImageMapper;
import org.jeecg.modules.kuaishou.service.IKuaiShouImageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.List;

/**
 * @Description: 快手-本地图片保存
 * @Author: jeecg-boot
 * @Date: 2019-08-14
 * @Version: V1.0
 */
@Service
public class KuaiShouImageServiceImpl extends ServiceImpl<KuaiShouImageMapper, KuaiShouImage> implements IKuaiShouImageService {

    @Autowired
    private KuaiShouImageMapper imageMapper;

    @Override
    public void insert(JSONObject requestJson) {
        JSONArray imageArr = requestJson.getJSONArray("localImageUrl");
        if (!Check.isNull(imageArr)) {
            for (int i = 0; i < imageArr.size(); i++) {
                KuaiShouImage image = new KuaiShouImage();
                String imageUrl = imageArr.get(i).toString();
                String fileName = imageUrl.substring(imageUrl.lastIndexOf("/") + 1);
                String fileNameStr = null;
                if (!Check.isNull(fileName)) {
                    String[] split = fileName.split("-");
                    fileNameStr = split[0];
                }
                image.setImageName(AesEncryptUtil.getURLDecoderString(fileNameStr));
                image.setLoginId(requestJson.getString("loginId"));
                image.setMaterialType(requestJson.getString("materialType"));
                image.setLocalUrl(KuaishouInterfaceConstant.HTTPS_PREFIX + imageUrl);
                imageMapper.insert(image);

            }

        }


    }

    @Override
    public String localInsert(String url, JSONObject requestJson) {
        String imageUrl = "";
        if (!Check.isNull(url)) {
            KuaiShouImage image = new KuaiShouImage();

            String fileName = url.substring(url.lastIndexOf("/") + 1);
            String fileNameStr = null;
            if (!Check.isNull(fileName)) {
                String[] split = fileName.split("-");
                fileNameStr = split[0];
            }
            image.setLoginId(requestJson.getString("loginId"));
            image.setImageName(AesEncryptUtil.getURLDecoderString(fileNameStr));
            image.setMaterialType(requestJson.getString("videoUploadType"));
            imageUrl = KuaishouInterfaceConstant.HTTPS_PREFIX + url;
            image.setLocalUrl(imageUrl);
            imageMapper.insert(image);


        }

        return imageUrl;
    }

    @Override
    public JSONArray selectImageList(QueryWrapper<KuaiShouImage> queryWrapper) {
        List<KuaiShouImage> kuaiShouImages = imageMapper.selectList(queryWrapper);
        JSONArray jsonArray = new JSONArray();
        if (!Check.isNull(kuaiShouImages)) {
            for (KuaiShouImage image : kuaiShouImages) {
                if (!Check.isNull(image)) {
                    JSONObject json = new JSONObject();
                    json.put("id", image.getId());
                    json.put("src", image.getLocalUrl());
                    json.put("msrc", image.getLocalUrl());
                    json.put("w", 600);
                    json.put("h", 400);
                    json.put("name", image.getImageName());
                    jsonArray.add(json);
                }
            }
        }

        return jsonArray;
    }
}
